@@ -1,6 +1,7 @@
 
 
-## Copyright 2015-2016 Tom Brown (FIAS), Jonas Hoersch (FIAS)
+## Copyright 2015-2019 Tom Brown (KIT, FIAS), Jonas Hoersch (KIT,
+## FIAS), David Schlachtberger (FIAS)
 
 ## This program is free software; you can redistribute it and/or
 ## modify it under the terms of the GNU General Public License as
@@ -21,14 +22,12 @@
 Grid calculation library.
 """
 
+from . import components, descriptors
+from . import (pf, opf, opt, plot, networkclustering, io, contingency, geo,
+               stats, linopf, linopt)
 
-from __future__ import absolute_import
+from .components import Network, SubNetwork
 
-from .components import Network, Bus, Load, Generator, ShuntImpedance, Line, Transformer, Converter, TransportLink, Link, SubNetwork, Branch, OnePort
-
-from . import components, pf, opf, plot, networkclustering, io, contingency, geo
-
-
-__version__ = "0.5.0"
-__author__ = "Tom Brown (FIAS), Jonas Hoersch (FIAS), David Schlachtberger (FIAS)"
-__copyright__ = "Copyright 2015-2016 Tom Brown (FIAS), Jonas Hoersch (FIAS), David Schlachtberger (FIAS), GNU GPL 3"
+__version__ = "0.17.0"
+__author__ = "Tom Brown (KIT, FIAS), Jonas Hoersch (KIT, FIAS), Fabian Hofmann (FIAS), Fabian Neumann (KIT), David Schlachtberger (FIAS)"
+__copyright__ = "Copyright 2015-2020 Tom Brown (KIT, FIAS), Jonas Hoersch (KIT, FIAS), Fabian Hofmann (FIAS), Fabian Neumann (KIT), David Schlachtberger (FIAS), GNU GPL 3"
