@@ -1,4 +1,4 @@
-## Copyright 2016 Tom Brown (FIAS)
+## Copyright 2016-2017 Tom Brown (FIAS)
 
 ## This program is free software; you can redistribute it and/or
 ## modify it under the terms of the GNU General Public License as
@@ -17,29 +17,28 @@
 """
 
 
-# make the code as Python 3 compatible as possible
-from __future__ import print_function, division, absolute_import
-
-
 __author__ = "Tom Brown (FIAS)"
-__copyright__ = "Copyright 2016 Tom Brown (FIAS), GNU GPL 3"
+__copyright__ = "Copyright 2016-2017 Tom Brown (FIAS), GNU GPL 3"
 
 
-from scipy.sparse import issparse, csr_matrix, csc_matrix, hstack as shstack, vstack as svstack
+from scipy.sparse import issparse, csr_matrix, csc_matrix, hstack as shstack
 
-from numpy import r_, ones, zeros, newaxis
+from numpy import r_, ones, zeros
+
+import logging
+logger = logging.getLogger(__name__)
 
 import numpy as np
 import pandas as pd
 
 import collections
 
-from .pf import calculate_PTDF
+from .pf import calculate_PTDF, _as_snapshots
 
 from .opt import l_constraint
 
 
-def calculate_BODF(sub_network,verbose=True,skip_pre=False):
+def calculate_BODF(sub_network, skip_pre=False):
     """
     Calculate the Branch Outage Distribution Factor (BODF) for
     sub_network.
@@ -58,14 +57,16 @@ def calculate_BODF(sub_network,verbose=True,skip_pre=False):
     Parameters
     ----------
     sub_network : pypsa.SubNetwork
-    verbose: bool, default True
-    skip_pre: bool, default False
+    skip_pre : bool, default False
         Skip the preliminary step of computing the PTDF.
 
+    Examples
+    --------
+    >>> sub_network.caculate_BODF()
     """
 
     if not skip_pre:
-        calculate_PTDF(sub_network,verbose)
+        calculate_PTDF(sub_network)
 
     num_branches = sub_network.PTDF.shape[0]
 
@@ -80,7 +81,7 @@ def calculate_BODF(sub_network,verbose=True,skip_pre=False):
     np.fill_diagonal(sub_network.BODF,-1)
 
 
-def network_lpf_contingency(network,snapshots=None,branch_outages=None,verbose=True,skip_pre=True):
+def network_lpf_contingency(network, snapshots=None, branch_outages=None):
     """
     Computes linear power flow for a selection of branch outages.
 
@@ -88,32 +89,27 @@ def network_lpf_contingency(network,snapshots=None,branch_outages=None,verbose=T
     ----------
     snapshots : list-like|single snapshot
         A subset or an elements of network.snapshots on which to run
-        the power flow, defaults to [now]
+        the power flow, defaults to network.snapshots
         NB: currently this only works for a single snapshot
     branch_outages : list-like
         A list of passive branches which are to be tested for outages.
         If None, it's take as all network.passive_branches_i()
-    verbose: bool, default True
-    skip_pre: bool, default False
-        Skip the preliminary steps of computing topology, calculating
-        dependent values and finding bus controls.
-    now : object
-        Deprecated: A member of network.snapshots on which to run the
-        power flow, defaults to network.now
 
     Returns
     -------
     p0 : pandas.DataFrame
         num_passive_branch x num_branch_outages DataFrame of new power flows
 
+    Examples
+    --------
+    >>> network.lpf_contingency(snapshot, branch_outages)
     """
 
-    from .components import passive_branch_types
-
     if snapshots is None:
-        snapshot = network.now
-    elif isinstance(snapshots, collections.Iterable):
-        print("Apologies LPF contingency, this only works for single snapshots at the moment, taking the first snapshot.")
+        snapshots = network.snapshots
+
+    if isinstance(snapshots, collections.Iterable):
+        logger.warning("Apologies LPF contingency, this only works for single snapshots at the moment, taking the first snapshot.")
         snapshot = snapshots[0]
     else:
         snapshot = snapshots
@@ -130,21 +126,21 @@ def network_lpf_contingency(network,snapshots=None,branch_outages=None,verbose=T
 
     p0_base = pd.Series(index=passive_branches.index)
 
-    for typ in passive_branch_types:
-        pnl = getattr(network,typ.list_name + "_t")
-        p0_base[typ.__name__] = pnl.p0.loc[network.now]
+    for c in network.passive_branch_components:
+        pnl = network.pnl(c)
+        p0_base[c] = pnl.p0.loc[snapshot]
 
     for sn in network.sub_networks.obj:
         sn._branches = sn.branches()
-        sn.calculate_BODF(verbose)
+        sn.calculate_BODF()
 
     p0 = pd.DataFrame(index=passive_branches.index)
 
     p0["base"] = p0_base
 
     for branch in branch_outages:
-        if type(branch) is not tuple and verbose:
-            print("No type given for {}, assuming it is a line".format(branch))
+        if not isinstance(branch, tuple):
+            logger.warning("No type given for {}, assuming it is a line".format(branch))
             branch = ("Line",branch)
 
         sn = network.sub_networks.obj[passive_branches.sub_network[branch]]
@@ -160,7 +156,9 @@ def network_lpf_contingency(network,snapshots=None,branch_outages=None,verbose=T
 
 
 
-def network_sclopf(network,snapshots=None,branch_outages=None,solver_name="glpk",verbose=True,skip_pre=False,solver_options={},keep_files=False,formulation="angles",ptdf_tolerance=0.):
+def network_sclopf(network, snapshots=None, branch_outages=None, solver_name="glpk",
+                   skip_pre=False, extra_functionality=None, solver_options={},
+                   keep_files=False, formulation="angles", ptdf_tolerance=0.):
     """
     Computes Security-Constrained Linear Optimal Power Flow (SCLOPF).
 
@@ -169,35 +167,47 @@ def network_sclopf(network,snapshots=None,branch_outages=None,solver_name="glpk"
     Parameters
     ----------
     snapshots : list or index slice
-        A list of snapshots to optimise, must be a subset of network.snapshots, defaults to network.now
+        A list of snapshots to optimise, must be a subset of
+        network.snapshots, defaults to network.snapshots
     branch_outages : list-like
         A list of passive branches which are to be tested for outages.
         If None, it's take as all network.passive_branches_i()
     solver_name : string
-        Must be a solver name that pyomo recognises and that is installed, e.g. "glpk", "gurobi"
-    verbose: bool, default True
-    skip_pre: bool, default False
-        Skip the preliminary steps of computing topology, calculating dependent values and finding bus controls.
+        Must be a solver name that pyomo recognises and that is
+        installed, e.g. "glpk", "gurobi"
+    skip_pre : bool, default False
+        Skip the preliminary steps of computing topology, calculating
+        dependent values and finding bus controls.
+    extra_functionality : callable function
+        This function must take two arguments
+        `extra_functionality(network,snapshots)` and is called after
+        the model building is complete, but before it is sent to the
+        solver. It allows the user to add/change constraints and
+        add/change the objective function.
     solver_options : dictionary
         A dictionary with additional options that get passed to the solver.
         (e.g. {'threads':2} tells gurobi to use only 2 cpus)
     keep_files : bool, default False
-        Keep the files that pyomo constructs from OPF problem construction, e.g. .lp file - useful for debugging
+        Keep the files that pyomo constructs from OPF problem
+        construction, e.g. .lp file - useful for debugging
     formulation : string
-        Formulation of the linear power flow equations to use; must be one of ["angles","cycles","kirchoff","ptdf"]
+        Formulation of the linear power flow equations to use; must be
+        one of ["angles","cycles","kirchoff","ptdf"]
     ptdf_tolerance : float
-        Value below which PTDF entries are ignored
 
     Returns
     -------
     None
+
+    Examples
+    --------
+    >>> network.sclopf(network, branch_outages)
     """
 
     if not skip_pre:
         network.determine_network_topology()
 
-    if snapshots is None:
-        snapshots = [network.now]
+    snapshots = _as_snapshots(network, snapshots)
 
     passive_branches = network.passive_branches()
 
@@ -208,10 +218,11 @@ def network_sclopf(network,snapshots=None,branch_outages=None,solver_name="glpk"
 
     for sn in network.sub_networks.obj:
 
-        sn.calculate_BODF(verbose)
+        sn.calculate_BODF()
 
         sn._branches = sn.branches()
         sn._branches["_i"] = range(sn._branches.shape[0])
+
         sn._extendable_branches = sn._branches[sn._branches.s_nom_extendable]
         sn._fixed_branches = sn._branches[~ sn._branches.s_nom_extendable]
 
@@ -224,24 +235,24 @@ def network_sclopf(network,snapshots=None,branch_outages=None,solver_name="glpk"
         flow_lower = {}
 
         for branch in branch_outages:
-            if type(branch) is not tuple and verbose:
-                print("No type given for {}, assuming it is a line".format(branch))
+            if type(branch) is not tuple:
+                logger.warning("No type given for {}, assuming it is a line".format(branch))
                 branch = ("Line",branch)
 
-            sub = network.sub_networks.obj[passive_branches.sub_network[branch]]
+            sub = network.sub_networks.at[passive_branches.at[branch,"sub_network"],"obj"]
 
             branch_i = sub._branches.at[branch,"_i"]
 
             branch_outage_keys.extend([(branch[0],branch[1],b[0],b[1]) for b in sub._branches.index])
 
-            flow_upper.update({(branch[0],branch[1],b[0],b[1],sn) : [[(1,network.model.passive_branch_p[b[0],b[1],sn]),(sub.BODF[sub._branches.at[b,"_i"],branch_i],network.model.passive_branch_p[branch[0],branch[1],sn])],"<=",sub._fixed_branches.s_nom[b]] for b in sub._fixed_branches.index for sn in snapshots})
+            flow_upper.update({(branch[0],branch[1],b[0],b[1],sn) : [[(1,network.model.passive_branch_p[b[0],b[1],sn]),(sub.BODF[sub._branches.at[b,"_i"],branch_i],network.model.passive_branch_p[branch[0],branch[1],sn])],"<=",sub._fixed_branches.at[b,"s_nom"]] for b in sub._fixed_branches.index for sn in snapshots})
 
-            flow_upper.update({(branch[0],branch[1],b[0],b[1],sn) : [[(1,network.model.passive_branch_p[b[0],b[1],sn]),(sub.BODF[sub._branches.at[b,"_i"],branch_i],network.model.passive_branch_p[branch[0],branch[1],sn]),(-1,network.model.branch_s_nom[b[0],b[1]])],"<=",0] for b in sub._extendable_branches.index for sn in snapshots})
+            flow_upper.update({(branch[0],branch[1],b[0],b[1],sn) : [[(1,network.model.passive_branch_p[b[0],b[1],sn]),(sub.BODF[sub._branches.at[b,"_i"],branch_i],network.model.passive_branch_p[branch[0],branch[1],sn]),(-1,network.model.passive_branch_s_nom[b[0],b[1]])],"<=",0] for b in sub._extendable_branches.index for sn in snapshots})
 
 
-            flow_lower.update({(branch[0],branch[1],b[0],b[1],sn) : [[(1,network.model.passive_branch_p[b[0],b[1],sn]),(sub.BODF[sub._branches.at[b,"_i"],branch_i],network.model.passive_branch_p[branch[0],branch[1],sn])],">=",-sub._fixed_branches.s_nom[b]] for b in sub._fixed_branches.index for sn in snapshots})
+            flow_lower.update({(branch[0],branch[1],b[0],b[1],sn) : [[(1,network.model.passive_branch_p[b[0],b[1],sn]),(sub.BODF[sub._branches.at[b,"_i"],branch_i],network.model.passive_branch_p[branch[0],branch[1],sn])],">=",-sub._fixed_branches.at[b,"s_nom"]] for b in sub._fixed_branches.index for sn in snapshots})
 
-            flow_upper.update({(branch[0],branch[1],b[0],b[1],sn) : [[(1,network.model.passive_branch_p[b[0],b[1],sn]),(sub.BODF[sub._branches.at[b,"_i"],branch_i],network.model.passive_branch_p[branch[0],branch[1],sn]),(1,network.model.branch_s_nom[b[0],b[1]])],">=",0] for b in sub._extendable_branches.index for sn in snapshots})
+            flow_lower.update({(branch[0],branch[1],b[0],b[1],sn) : [[(1,network.model.passive_branch_p[b[0],b[1],sn]),(sub.BODF[sub._branches.at[b,"_i"],branch_i],network.model.passive_branch_p[branch[0],branch[1],sn]),(1,network.model.passive_branch_s_nom[b[0],b[1]])],">=",0] for b in sub._extendable_branches.index for sn in snapshots})
 
 
         l_constraint(network.model,"contingency_flow_upper",flow_upper,branch_outage_keys,snapshots)
@@ -249,7 +260,12 @@ def network_sclopf(network,snapshots=None,branch_outages=None,solver_name="glpk"
 
         l_constraint(network.model,"contingency_flow_lower",flow_lower,branch_outage_keys,snapshots)
 
+        if extra_functionality is not None:
+            extra_functionality(network, snapshots)
 
     #need to skip preparation otherwise it recalculates the sub-networks
 
-    network.lopf(snapshots=snapshots,solver_name=solver_name,verbose=verbose,skip_pre=True,extra_functionality=add_contingency_constraints,solver_options=solver_options,keep_files=keep_files,formulation=formulation,ptdf_tolerance=ptdf_tolerance)
+    network.lopf(snapshots=snapshots, solver_name=solver_name, skip_pre=True,
+                 extra_functionality=add_contingency_constraints,
+                 solver_options=solver_options, keep_files=keep_files,
+                 formulation=formulation, ptdf_tolerance=ptdf_tolerance)
